import numpy as np
import torch
import torchvision
import torchvision.transforms as transforms
import torch.nn as nn


#Flatten 
#https://pytorch.org/docs/stable/generated/torch.nn.Flatten.html

rand_img = torch.rand([30, 3, 40, 40])
print(rand_img.shape)
flat_img = torch.flatten(rand_img)
print(flat_img.shape)

flat_layer = nn.Flatten()
y = flat_layer(rand_img)
print(y.shape)

class Model_0_linear(nn.Module):
    def __init__(self, input_size, hidden_size, output_size):
        super().__init__()
        self.layers = nn.Sequential(
            nn.Flatten(),
            nn.Linear(input_size, hidden_size),
            nn.Linear(hidden_size, output_size)
        )
        
    def forward(self, x):
        return self.layers(x)


class Model_0_nonlinear(nn.Module):
    def __init__(self, input_size, hidden_size, output_size):
        super().__init__()
        self.layers = nn.Sequential(
            nn.Flatten(),
            nn.Linear(input_size, hidden_size),
            nn.ReLU(),
            nn.Linear(hidden_size, output_size),
            nn.ReLU() # and so on 
        )

    def forward(self, x):
        return self.layers(x)         
     

rand_img = torch.rand([4, 3, 40, 40])
input_size = 3*40*40
hidden_size = 32
output_size = 4

net = Model_0_linear(input_size, hidden_size, output_size).to("cpu")
print(net)
print(net(rand_img))

net = Model_0_nonlinear(input_size, hidden_size, output_size).to("cpu")
print(net)
print(net(rand_img))


